function loadTable(output, keys, value, depth)
	depth = depth or 1
	output = output or {}
	if depth<=#keys then
		output[keys[depth]] = loadTable(output[keys[depth]],keys,value,depth+1)
		return output
	else
		return value
	end
end

function loadMap(file,world)
	world = world or phyz.newWorld()
	--Reset map
	map = {
		background = {255,255,255},
		scale = 64, --Shouldn't need to change this, it is used for setMeter()
		world = world,
		spawn = {
		x = 0,
		y = 0,
		},
		player = {
		},
		phyzObjs = {}, --Physic objects
		staticObjs = {}, --Other objects like signs/exits etc
	}
	if not love.filesystem.exists(file) then
		return false
	end
	
	for v in love.filesystem.lines(file) do
		local kv = {}
		if string.sub(v,1,1)~="#" then
			string.gsub(v, "([%P.]+)", function (w)
				if #kv==0 then
					table.insert(kv, w)
				end
			end)
			kv[2] = string.sub(v,string.find(v,":")+1)
			local key = {}
			string.gsub(kv[1], "(%P+)", function (w)
				if w=="true" then w = true end
				if w=="false" then w = false end
				if tonumber(w) then w = tonumber(w) end
				table.insert(key, w)
			end)
			if kv[2]=="true" then kv[2] = true end
			if kv[2]=="false" then kv[2] = false end
			if tonumber(kv[2]) then kv[2] = tonumber(kv[2]) end
			loadTable(map,key,kv[2])
		end
	end

	for i=1,#map.phyzObjs do --Convert the raw text and number version into phyzHook compatible data
		local args = map.phyzObjs[i]
		if not
			pcall(function()
				map.phyzObjs[i] = world["new"..map.phyzObjs[i].t](world,unpack(args))
			end) 
		then error("Attempted creating phyzobject ("..map.phyzObjs[i].t..")") end
		if args["texture"] then
			map.phyzObjs[i].texture = args.texture
		end
		if args["textureType"] then
			map.phyzObjs[i].textureType = args.textureType
		end
		if args["gravity"] then
			world:setGravity(map.phyzObjs[i],tonumber(args.gravity[1]),tonumber(args.gravity[2]))
		end
		if args["selfRighting"] then
			map.phyzObjs[i].body:setFixedRotation(true)
		end
		map.phyzObjs[i].onGround = false
		loadTexture(map.phyzObjs[i])
	end
	
	for i=1,#map.staticObjs do
		if map.staticObjs[i].image then
			loadStaticTexture(map.staticObjs[i].image)
		end
		if map.staticObjs[i].text then
			for k,v in pairs(_keys) do --good job keyBinder leaves the keys globally visible
				map.staticObjs[i].text = string.gsub(map.staticObjs[i].text,"$KEY_"..string.upper(k),string.upper(v))
			end
		end
	end
	
	map.player = world:newCircle(map.spawn.x,map.spawn.y,25,"dynamic")
	map.player.texture = "assets/player.png"
	map.player.textureType = "scale"
	map.player.selfRighting = true
	map.player.body:setFixedRotation(true)
	map.player.fixture:setUserData("Player")
	map.player.onGround = false
	loadTexture(map.player)
	
	love.graphics.setBackgroundColor(map.background)
	
	world = phyz.newWorld(map.scale)
	
	preDraw() --Set up the textures for the level
	
	return world
end