gameSettings = { --The actual settings values for use in the code
["Text speed"] = 20,
["Compass X"] = 0,
["Compass Y"] = 0,
["Fullscreen"] = false,
["Vsync"] = false,
}

--Button Types:
--Reel: 	{Name, "reel", {Values}, Current Value, Description}
--Percent:	{Name, "percent", Value, Description}

local settings = {
{"Text speed", "reel", {5,10,15,20,30,40,50,60,70,80,90,100}, 1, "How fast text scrolls in pixels per second"},
{"Compass X", "percent", 0, "The X position of the compass as a percentage of the screen width"},
{"Compass Y", "percent", 0, "The Y position of the compass as a percentage of the screen height"},
{"Fullscreen", "bool", false, "Run the window in fullscreen, The window is letterboxed to fit a 4:3 ratio"},
{"Vsync", "bool", false, "Use Vsync to cap the framerate at the monitor's refresh rate"},
}

local controls = {
{"Jump", "up", "Makes the player jump from surfaces and drift up in the void"},
{"Fall", "down", "Makes the player fall faster in gravity fields and drift down in the void"},
{"Left", "left", "Makes the player drift left"},
{"Right", "right", "Makes the player drift right"},
{"Fields", "f", "Toggles whether you can see gravity fields"},
}

local selectedControl = nil

local selectedSetting = nil
local customValue = nil

local scrollDis = 0
local maxScroll = 0

function drawOptions()
	love.graphics.setBackgroundColor(100,0,0)
	local down = 40 - scrollDis
	love.graphics.print("Controls",20,down-30)
	for i=1,#controls do
		local k,v,d = controls[i][1], controls[i][2], controls[i][3] or "" -- The key, value and description for each bind
		love.graphics.setColor(10,100,200)
		love.graphics.rectangle("fill",20,down,200,50)
		love.graphics.setColor(255,255,255)
		love.graphics.print(d,230,down+15)
		if i==selectedControl then v = "???" end
		if v==" " then v = "space" end --Space has a stupid key name
		love.graphics.printf(k..": "..v,20,down+15,200,"center")
		down = down+60
	end
	love.graphics.print("Settings",20,down)
	down = down + 30
	for i=1,#settings do
		local name = settings[i][1]
		local Type = settings[i][2]
		if Type=="reel" then
			love.graphics.setColor(10,100,200)
			love.graphics.rectangle("fill",20,down,200,50)
			love.graphics.setColor(255,255,255)
			love.graphics.print(settings[i][5],230,down+15)
			love.graphics.printf(name..": "..settings[i][3][settings[i][4]],20,down+15,200,"center")
		end
		if Type=="percent" then
			love.graphics.setColor(10,100,200)
			love.graphics.rectangle("fill",20,down,200,50)
			love.graphics.setColor(255,255,255)
			love.graphics.print(settings[i][4],230,down+15)
			if selectedSetting==i then
				love.graphics.printf(name..": "..customValue,20,down+15,200,"center")
			else
				love.graphics.printf(name..": "..settings[i][3].."%",20,down+15,200,"center")
			end
		end
		if Type=="bool" then
			love.graphics.setColor(10,100,200)
			love.graphics.rectangle("fill",20,down,200,50)
			love.graphics.setColor(255,255,255)
			love.graphics.print(settings[i][4],230,down+15)
			love.graphics.printf(name..": "..tostring(settings[i][3]),20,down+15,200,"center")
		end
		down = down+60
	end
	down = down + 20
	love.graphics.setColor(10,100,200)
	love.graphics.rectangle("fill",325,down,150,50)
	love.graphics.setColor(255,255,255)
	love.graphics.printf("Save",325,down+15,150,"center")
	if down+60+scrollDis>love.graphics.getHeight() then
		maxScroll = (down+60+scrollDis) - love.graphics.getHeight() --Update if resolution changes
		love.graphics.rectangle("fill",love.graphics.getWidth()-15,scrollDis,10,love.graphics.getHeight()/(down+50) * love.graphics.getHeight())
	end
end

function updateOptions(dt)
	if love.keyboard.isDown("escape") then
		state = "title"
	end
end

function keyPressedOptions(k,u)
	if selectedControl then
		if k then
			controls[selectedControl][2] = k
			selectedControl = nil
		end
	end
	if selectedSetting then
		if settings[selectedSetting][2] == "percent" then
			if tonumber(k) then
				customValue = math.min(100,math.max(tonumber(tostring(customValue)..k),0))
			end
			if k=="backspace" then
				customValue = tonumber(tostring(customValue):sub(1,-2)) or 0
			end
			if k=="return" then
				settings[selectedSetting][3] = customValue
				selectedSetting = nil
				customValue = nil
			end
		end
	end
end

function mousePressedOptions(x,y,b)
	if b=="wd" and maxScroll>0 then
		scrollDis = math.min(scrollDis + 10,maxScroll)
	end
	if b=="wu" and maxScroll>0 then
		scrollDis = math.max(scrollDis - 10,0)
	end
	if not (selectedControl or selectedSetting) and b=="l" then
		local down = 40 - scrollDis
		for i=1,#controls do
			local k,v = controls[i][1], controls[i][2]
			if x>20 and x<220 and y>down and y<down+50 then
				selectedControl = i
			end
			down = down+60
		end
		down = down + 30
		for i=1,#settings do
			if x>20 and x<220 and y>down and y<down+50 then
				if settings[i][2]=="reel" then
					settings[i][4] = 1 + (settings[i][4])%#settings[i][3]
				end
				if settings[i][2]=="percent" then
					selectedSetting = i
					customValue = 0
				end
				if settings[i][2]=="bool" then
					settings[i][3] = not settings[i][3]
				end
			end
			down = down+60
		end
		down = down + 20
		if x>325 and x<475 and y>down and y<down+50 then
			saveControls()
			saveSettings()
			state = "title"
		end
	end
end

function saveControls()
	if love.filesystem.exists("controls.txt") then love.filesystem.remove("controls.txt") end
	for i=1,#settings do
		local s = settings[i]
		if s[2]=="reel" then
			gameSettings[s[1]] = s[3][s[4]]
		end
	end--Save the settings to RAM first
	
	local file = love.filesystem.newFile("controls.txt")
	file:open("w")
	local nl = false
	for i=1,#controls do
		if nl then file:write("\n") else nl = true end
		file:write(controls[i][1]..":"..controls[i][2])
	end
end

function loadControls()
	if not love.filesystem.exists("controls.txt") then --Set defaults if none loaded
		saveControls()
	else
		for line in love.filesystem.lines("controls.txt") do
			local i = string.find(line,":")
			for x=1,#controls do
				if controls[x][1] == string.sub(line,1,i-1) then --Only load valid options
					controls[x][2] = string.sub(line,i+1)
				end
			end
		end
	end
	
	for i=1,#controls do
		local k,v = controls[i][1], controls[i][2]
		binds.bind(v,k) --Binds the keys from the controls table
	end
end

loadControls() --Load the controls the player has bound

function saveSettings()
	if love.filesystem.exists("settings.txt") then love.filesystem.remove("settings.txt") end
	
	local file = love.filesystem.newFile("settings.txt")
	file:open("w")
	local nl = false
	for i=1,#settings do
		if nl then file:write("\n") else nl = true end
		local s = settings[i]
		if s[2]=="reel" then
			gameSettings[s[1]] = s[3][s[4]] --Save to RAM as well
			file:write(s[1]..":"..s[4])
		end
		if s[2]=="percent" then
			gameSettings[s[1]] = s[3]
			file:write(s[1]..":"..s[3])
		end
		if s[2]=="bool" then
			gameSettings[s[1]] = s[3]
			file:write(s[1]..":"..tostring(s[3]))
		end
	end
	local _,_,f,v = love.graphics.getMode()
	if f~=gameSettings["Fullscreen"] or v~=gameSettings["Vsync"] then
		if gameSettings["Fullscreen"] then
			love.graphics.setMode(0,0,gameSettings["Fullscreen"],gameSettings["Vsync"])
		else
			love.graphics.setMode(800,600,gameSettings["Fullscreen"],gameSettings["Vsync"])
		end
	end
end

function loadSettings()
	if not love.filesystem.exists("settings.txt") then --Set defaults if none loaded
		saveSettings()
	else
		for line in love.filesystem.lines("settings.txt") do
			local i = string.find(line,":")
			for x=1,#settings do
				if i and settings[x][1] == string.sub(line,1,i-1) then
					if settings[x][2] == "reel" then
						settings[x][4] = tonumber(string.sub(line,i+1))
					end
					if settings[x][2] == "percent" then
						settings[x][3] = tonumber(string.sub(line,i+1))
					end
					if settings[x][2] == "bool" then
						settings[x][3] = (string.sub(line,i+1))~="false"
					end
				end
			end
		end
	end
	for i=1,#settings do
		local s = settings[i]
		if s[2]=="reel" then
			gameSettings[s[1]] = s[3][s[4]]
		end
		if s[2]=="percent" then
			gameSettings[s[1]] = s[3]
		end
		if s[2]=="bool" then
			gameSettings[s[1]] = s[3]
		end
	end
	--Set up the display to match settings
	local _,_,f,v = love.graphics.getMode()
	if f~=gameSettings["Fullscreen"] or v~=gameSettings["Vsync"] then
		if gameSettings["Fullscreen"] then
			love.graphics.setMode(0,0,gameSettings["Fullscreen"],gameSettings["Vsync"])
		else
			love.graphics.setMode(800,600,gameSettings["Fullscreen"],gameSettings["Vsync"])
		end
	end
end

loadSettings()