local phyzhook = {}

local phy = {objects = {}}

--Very useful and small function to turn multiple return values into a table
function pack(...)
	local arg = {...}
	return arg
end

--Reduce referencing
local remove,sqrt = table.remove,math.sqrt
local lphy = love.physics

--===========================================================================================================New World
function phyzhook.newWorld(sc)
	w = {}
	setmetatable(w, phy) --Create a copy of "phy" which is the master copy of a world
	phy.__index = phy
	w.world = lphy.newWorld() --VERY important not to have default gravity because of the whole basis of the game
	love.physics.setMeter(sc or 64)
	return w
end
--===========================================================================================================New Polygon
function phy:newPolygon(x,y,...)
	local arg = {...}
	local typ = arg[#arg-2]
	local vx,vy = 0,0
	local lowx,lowy = nil,nil
	local highx,highy = nil,nil
	if type(arg[#arg])=="string" then --Determine whether the function specifies initial velocity after the object type
		typ = arg[#arg]
	else
		vx = arg[#arg-1]
		vy = arg[#arg]
		remove(arg,#arg)
		remove(arg,#arg)
	end
	remove(arg,#arg) --Remove extra arguments to allow passing the arguments into love.physics.newPolygonShape
	--Calculate the REAL centre (Without this the textures get messed up) by finding the highest and lowest values...
	for i=1,#arg,2 do
		lowx = (not lowx and arg[i]) or (arg[i]<lowx and arg[i] or lowx)
		highx = (not highx and arg[i]) or (arg[i]>highx and arg[i] or highx)
		
		lowy = (not lowy and arg[i+1]) or (arg[i+1]<lowy and arg[i+1] or lowy)
		highy = (not highy and arg[i+1]) or (arg[i+1]>highy and arg[i+1] or highy)
	end
	--...then getting the centre of them
	local cx = lowx + (highx-lowx)/2
	local cy = lowy + (highy-lowy)/2
	--Move the points to be relative to 0,0
	for i=1,#arg,2 do
		arg[i] = arg[i]-cx
		arg[i+1] = arg[i+1]-cy
	end
	local poly = {}
	poly.type = "poly" --Allows me to determine which calculation to apply without having to use Box2D functions
	poly.shape = lphy.newPolygonShape(unpack(arg))
	poly.body = lphy.newBody(self.world,x,y,typ)
	poly.fixture = lphy.newFixture(poly.body,poly.shape)
	poly.body:applyForce(vx,vy) --Apply initial velocity
	self:setGravity(poly)
	self.objects[#self.objects+1] = poly --Add object to the list so applyGravity can iterate over them
	if typ=="dynamic" then end
	return poly --Give it back to the user for manipulation
end
--===========================================================================================================New Rectangle
function phy:newRectangle(x,y,width,height,rotation,type,initVelX,initVelY)
	initVelX = initVelX or 0
	initVelY = initVelY or 0
	local poly = {}
	poly.type = "poly" --Allows me to determine which calculation to apply without having to use Box2D functions
	poly.shape = lphy.newRectangleShape(0,0,width,height,rotation)
	poly.body = lphy.newBody(self.world,x,y,type)
	poly.fixture = lphy.newFixture(poly.body,poly.shape)
	poly.body:applyForce(initVelX,initVelY) --Apply initial velocity
	self:setGravity(poly)
	self.objects[#self.objects+1] = poly --Add object to the list so applyGravity can iterate over them
	return poly --Give it back to the user for manipulation
end
--===========================================================================================================New Circle
function phy:newCircle(x,y,r,type,initVelX,initVelY)
	initVelX = initVelX or 0
	initVelY = initVelY or 0
	local poly = {}
	poly.type = "circle" --Allows me to determine which calculation to apply without having to use Box2D functions
	poly.shape = lphy.newCircleShape(r)
	poly.body = lphy.newBody(self.world,x,y,type)
	poly.fixture = lphy.newFixture(poly.body,poly.shape)
	poly.body:applyForce(initVelX,initVelY) --Apply initial velocity
	self:setGravity(poly)
	self.objects[#self.objects+1] = poly --Add object to the list so applyGravity can iterate over them
	return poly --Give it back to the user for manipulation
end
--===========================================================================================================Add Gravity
function phy:setGravity(obj,strength,range)
	obj = obj or error("No object to set gravity to",2) --Hopefully this error is never shown to the user as the map loader should have fail-safes
	obj.gravityS = tonumber(strength) or obj.gravityS or 0
	obj.gravityR = tonumber(range) or obj.gravityR or 0
end

--===========================================================================================================Update World
function phy:update(dt)
	dt = math.min(1/30,dt) --Stop lag spikes messing up things
	for i=1,#self.objects do
		self:applyGravity(self.objects[i],dt) --Calculate gravity changes
	end
	for i=1,#map.staticObjs do
		local o = map.staticObjs[i]
		if o.type=="objective" then
			o.rot = (o.rot+dt)%6.28
			local x,y = map.player.body:getWorldCenter()
			if (x-o.x)^2 + (y-o.y)^2 < o.radius^2 then
				state = "title"
				resetTextures()
			end
		end
		if o.type=="text" then
			if not o.speed then
				o.progress = (o.progress+dt*gameSettings["Text speed"])%((love.graphics.getFont():getWidth(o.text)+180))
			else
				o.progress = (o.progress+dt*o.speed)%((love.graphics.getFont():getWidth(o.text)+180))
			end
			--Check the player is in range
			local px,py = map.player.body:getWorldCenter()
			if (px-o.x)^2 + (py-o.y)^2 > o.range^2 then
				o.progress = 0
			end
		end
	end
	self.world:update(dt) --Update the actual world
end
--===========================================================================================================Apply gravity
function phy:applyGravity(move_obj,dt)
	local compX,compY = 0,0 --Composite velocity
	move_obj.onGround = false
	move_obj.inVoid = true
	for i=1,#self.objects do
		if self.objects[i]~=move_obj and self.objects[i].gravityS~=0 then
			--Name some objects to save referencing
			grav = self.objects[i]
			--Move closest point
			local dis,x1,y1,x2,y2 = lphy.getDistance(move_obj.fixture,grav.fixture) --This saved a lot of code
			local diffX,diffY = x2-x1,y2-y1
			
			local gravity = grav.gravityS+move_obj.gravityS --Composite strength of gravity
			
			if dis<=grav.gravityR and dis>0 then
				local constant = 60*gravity*(1-(dis/grav.gravityR))
				local velX,velY = (constant/dis)*diffX,(constant/dis)*diffY
				move_obj.body:applyForce(velX,velY,x1,y1)
				compX = compX+velX
				compY = compY+velY
			end
			move_obj.onGround = dis<=2 or move_obj.onGround --Determine whether jumping is possible for player
			
			--Move centre
			local x,y = move_obj.body:getWorldCenter()
			local cx,cy = nil,nil
			local dis = nil
			
			if grav.type=="circle" then
				cx,cy = grav.body:getWorldCenter() --Cheat for finding the closest point, just add the radius to the range when checking
			else
				local points = pack(grav.body:getWorldPoints(grav.shape:getPoints()))
				for i=1,#points,2 do
					local p,p2 = {x = points[i],y = points[i+1]},{x = points[((i+1)%#points)+1],y = points[((i+2)%#points)+1]}
					local collx,colly,d = self:getClosestPointSegment(p.x,p.y,p2.x,p2.y,x,y)
					cx = ((not cx) and collx) or (dis<d and cx) or collx
					cy = ((not cy) and colly) or (dis<d and cy) or colly
					dis = ((not dis) and d) or (dis<d and dis) or d
				end
			end
			
			local diffX,diffY = cx-x,cy-y
			dis = dis or sqrt(diffX^2+diffY^2) --Find the distance to check if shape is in range
			if grav.type=="circle" then
				dis = dis-grav.shape:getRadius() --The aforementioned cheat, only in reverse for some reason
			end
			if dis>0 and dis<=grav.gravityR then
				move_obj.inVoid = false --Used to tell player if they should drift or jump when pressing jump key
				local constant = 200*gravity*(1-(dis/grav.gravityR))
				local velX = ((constant/dis)*diffX) --Normalise the velocity
				local velY = ((constant/dis)*diffY) --Normalise the velocity
				compX = compX+velX
				compY = compY+velY
				move_obj.body:applyForce(velX,velY)
			end
		end
		if move_obj.selfRighting and (compX~=0 or compY~=0) then
			move_obj.body:setAngle((3*math.pi/2+math.atan2(compY,compX))%(2*math.pi)) --Rotate the object to face "up" from the surface
		end
		if move_obj.inVoid and move_obj.fixture:getUserData()~="Player" then --Allow things to drift in space (not player though, too hard to control)
			move_obj.body:setLinearDamping(0)
		else
			move_obj.body:setLinearDamping(5) --5 feels like a good number to go with
		end
	end
end
--===========================================================================================================Get Closest Point On a Segment
function phy:getClosestPointSegment(startX,startY,endX,endY,pointX,pointY)
	--Calculate equations for the lines
	local grad1 = (endY-startY)/(endX-startX)
	local offset1 = endY-(grad1*endX)
	
	local grad2 = -1/grad1
	local offset2 = pointY-(grad2*pointX)
	
	local collX = (offset1-offset2)/(grad2-grad1)
	local collY = grad1*collX+offset1
	
	--Sort out errors involving 1/0
	collX = grad1==0 and pointX or collX 
	collY = grad1==0 and endY or collY
	
	collX = grad2==0 and endX or collX
	collY = grad2==0 and pointY or collY

	--Round values to be in range
	collX = ((endX>startX and collX>=endX) and endX) or ((endX<=startX and collX>=startX) and startX) or collX --Check if the x is too high
	collX = ((endX<startX and collX<=endX) and endX) or ((endX>=startX and collX<=startX) and startX) or collX --Check if the x is too low
	
	collY = ((endY>startY and collY>=endY) and endY) or ((endY<=startY and collY>=startY) and startY) or collY --Check if the y is too high
	collY = ((endY<startY and collY<=endY) and endY) or ((endY>=startY and collY<=startY) and startY) or collY --Check if the y is too low
	return collX,collY,sqrt((pointX-collX)^2+(pointY-collY)^2)
end
--===========================================================================================================Get Object Centre
function phy:getCenter(obj) --hacky, but works
	local typ = obj.body:getType()
	local vx,vy = obj.body:getLinearVelocity()
	local i = obj.body:getInertia()
	local a = obj.body:getAngularVelocity()
	obj.body:setType("static")
	local x,y = obj.body:getWorldCenter()
	obj.body:setType(typ)
	obj.body:setLinearVelocity(vx,vy)
	obj.body:setInertia(i)
	obj.body:setAngularVelocity(a)
	return x,y
end

return phyzhook --Turn it all into an API