--===========================================================================================================Variables
local textures = {} --The base textures manipulated to make phyzImages

local phyzImages = {} --The generated image of each phyzObject

local phyzFields = {} -- The gravity fields of each phyzObject

local DEFAULT_TEXTURE = love.graphics.newImage("assets/missing.png")

local compass = love.graphics.newImage("assets/compass.png") --When testing I kept losing track of up and down

local objective = love.graphics.newImage("assets/wormhole.png") --The objective for the level

local display = love.graphics.newImage("assets/display.png")

local displayBack = love.graphics.newImage("assets/displayBack.png")

local graph = love.graphics
local draw = love.graphics.draw

DRAW_FIELDS = false --Experimental rendering of gravity fields
--===========================================================================================================Load textures
function loadTexture(obj)
	if not obj.texture then return end
	if not pcall(function()
		local texture = obj.texture
		textures[texture] = textures[texture] or graph.newImage(texture)
		obj.texture = textures[texture]
	end) then
		local texture = obj.texture
		textures[texture] = DEFAULT_TEXTURE
		obj.texture = textures[texture]
	end
end
--===========================================================================================================Load static textures
function loadStaticTexture(path)
	if not pcall(function()
		textures[path] = textures[path] or graph.newImage(path)
	end) then
	textures[path] = DEFAULT_TEXTURE
	end
end
--===========================================================================================================Pre-draw phyzField
function drawField(index) --Index allows for potential changing of fields in game (maybe a puzzle element idea)
	local tx,ty,bx,by = map.phyzObjs[index].shape:computeAABB(0,0,0)
	local w,h = (bx-tx) + map.phyzObjs[index].gravityR*2,(by-ty) + map.phyzObjs[index].gravityR*2

	local data = graph.newCanvas(w,h)
	graph.setCanvas(data)
	if map.phyzObjs[index].type=="circle" then
		graph.setColor(0,0,150,150)
		graph.circle("fill",data:getWidth()/2,data:getHeight()/2,data:getWidth()/2)
		graph.setColor(255,255,255)
	else
		local stencil = graph.newStencil(function()
			local points = pack(map.phyzObjs[index].shape:getPoints())
			graph.translate(w/2,h/2)
			local cx = 0
			local cy = 0
			for i=1,#points do
				if i%2==0 then
					cy=cy+points[i]
				else
					cx = cx+points[i]
				end
			end
			cx = cx/(#points/2)
			cy = cy/(#points/2)
			for i=1,#points,2 do
				local startPoint = {x=points[i],y=points[i+1]}
				local endPoint = {x=points[(i+1)%#points+1],y=points[(i+1)%#points+2]}
				
				local dy = endPoint.y-startPoint.y
				local grad = -(endPoint.x-startPoint.x)/(endPoint.y-startPoint.y)
				
				local con = math.sqrt(1+grad^2)/map.phyzObjs[index].gravityR
				
				dy = (dy/math.abs(dy))
				
				local newStartPoint = {
				x = startPoint.x+dy/con,
				y = startPoint.y+dy*grad/con,
				}
				
				local newEndPoint = {
				x = endPoint.x+dy/con,
				y = endPoint.y+dy*grad/con,
				}
				
				local diff = (startPoint.y-cy)/math.abs(startPoint.y-cy)

				if math.abs(grad)==1/0 then --Damn dividing by 0 problems
					newStartPoint.x = startPoint.x
					newEndPoint.x = endPoint.x
					newStartPoint.y = startPoint.y+map.phyzObjs[index].gravityR*diff
					newEndPoint.y = endPoint.y+map.phyzObjs[index].gravityR*diff
				end

				graph.polygon("fill",startPoint.x,startPoint.y,newStartPoint.x,newStartPoint.y,newEndPoint.x,newEndPoint.y,endPoint.x,endPoint.y)
				graph.circle("fill",endPoint.x,endPoint.y,map.phyzObjs[index].gravityR,map.phyzObjs[index].gravityR*3)
			end
			graph.translate(-w/2,-h/2)
		end)
		
		graph.setStencil(stencil)
		graph.setColor(0,0,150,150)
		graph.rectangle("fill",0,0,w,h)
		graph.setColor(255,255,255)
		graph.setStencil()
	end
	graph.setCanvas()
	phyzFields[index] = graph.newImage(data:getImageData())
end
--===========================================================================================================Pre-draw images
function preDraw()
	-- res.scale(1)
	-- if res.letterBox==1 then
		-- graph.translate(-res.letterSize,0)
	-- else
		-- graph.translate(0,-res.letterSize)
	-- end
	-- res.setScissor()
	graph.clear()
	for i=1,#map.phyzObjs do
		drawField(i) --Set up the textures for the gravity fields
		local tex = map.phyzObjs[i].texture or DEFAULT_TEXTURE --Get a texture if none loaded
		if map.phyzObjs[i].textureType=="scale" then
			local x,y,x2,y2 = map.phyzObjs[i].shape:computeAABB(0,0,0)
			local w,h = x2-x,y2-y --Calculate the width and height of the image
			local data = graph.newCanvas(w,h) --Sorry non canvas users, I couldn't find a way around this
			local sten = graph.newStencil(function() --Create a stencil of the shape so the texture is the right shape
				if map.phyzObjs[i].type=="circle" then
					graph.circle("fill",w/2,h/2,w/2)
				else
					graph.translate(-w/2,-h/2)
					graph.polygon("fill",map.phyzObjs[i].shape:getPoints())
					graph.translate(w/2,h/2)
				end
			end)
			tex:setFilter("nearest","nearest")
			graph.setCanvas(data)
			graph.setStencil(sten)
			draw(tex,0,0,0,w/tex:getWidth(),h/tex:getHeight()) --Fill the area with a scaled version of the texture
			graph.setCanvas()
			graph.setStencil()
			phyzImages[i] = graph.newImage(data:getImageData())
		else
			local x,y,x2,y2 = map.phyzObjs[i].shape:computeAABB(0,0,0)
			local w,h = x2-x,y2-y --Calculate the width and height of the image
			local data = graph.newCanvas(w,h) --Sorry non canvas users, I couldn't find a way around this
			data:setFilter("nearest","nearest")
			graph.setCanvas(data)
			local sten = graph.newStencil(function() --Create a stencil of the shape so the texture is the right shape
				if map.phyzObjs[i].type=="circle" then
					graph.circle("fill",w/2,h/2,w/2)
				else
					graph.translate(w/2,h/2) 
					graph.polygon("fill",map.phyzObjs[i].shape:getPoints())
					graph.translate(-w/2,-h/2)
				end
			end)
			graph.setStencil(sten)
			for x=1,math.ceil(w/tex:getWidth()) do --Fill the area with the texture, repeating
				for y=1,math.ceil(h/tex:getHeight()) do
					draw(tex,(x-1)*tex:getWidth(),(y-1)*tex:getHeight())
				end
			end
			graph.setStencil()
			graph.setCanvas()
			phyzImages[i] = graph.newImage(data:getImageData()) --Save the canvas as an image
		end
	end
	-- res.scale(res.scaleRate)
	-- if res.letterBox==1 then
		-- graph.translate(res.letterSize,0)
	-- else
		-- graph.translate(0,res.letterSize)
	-- end
	-- graph.setScissor() -- resolution overrides this to make sure I can only draw in frame
end
--===========================================================================================================Draw level
function drawLevel()
	graph.push()
	local x,y = map.player.body:getWorldCenter()
	local w,h = graph.getWidth(),graph.getHeight()
	graph.translate(w/2, h/2)
    graph.rotate(-map.player.body:getAngle())
	graph.translate(-w/2,-h/2)
	graph.translate((w/2)-x,(h/2)-y)
	for i=1,#map.staticObjs do
		local o = map.staticObjs[i]
		if o.type=="image" then
			draw(textures[o.image],o.x,o.y,o.rot)
		end
		if o.type=="objective" then
			local width = objective:getWidth()
			draw(objective,o.x,o.y,map.staticObjs[i].rot,o.radius/(objective:getWidth()/2),o.radius/(objective:getHeight()/2),width/2,width/2)
		end
		if o.type=="text" then
			local f = graph.getFont()
			local text = o.text
			local offset = 0
			local last = text:len()
			local first = 1
			--Cycle through and find the last visible letter
			for i=1,text:len() do
				if o.progress-10<f:getWidth(text:sub(1,i)) then
					last = i
					break
				end
			end
			--Chop off letters that have passed the edge
			if o.progress>165 then
				local p = o.progress-165 --The extra distance the text extends off the left edge
				for i=1,o.text:len() do
					if f:getWidth(o.text:sub(1,i))<=p and i~=1 then --Check if the letter needs trimming
						offset = f:getWidth(o.text:sub(1,i-1))
						first = i --Trim it to be just the letters after the cut off point
					end
				end
			end
			graph.setColor(30,30,30)
			graph.draw(displayBack,o.x,o.y,o.rot,1,1,displayBack:getWidth()/2,displayBack:getHeight()/2)
			graph.setColor(255,255,255)
			graph.print(text:sub(first,last),o.x,o.y,o.rot,1,1,display:getWidth()/2 - 190 + o.progress - offset,display:getHeight()/2 - 15)
			draw(display,o.x,o.y,o.rot,1,1,display:getWidth()/2,display:getHeight()/2)
		end
	end
	for i=1,#map.phyzObjs do
		local body = map.phyzObjs[i].body --Save on referencing
		local shape = map.phyzObjs[i].shape
		local x,y = map.world:getCenter(map.phyzObjs[i])
		if DRAW_FIELDS then --Draw gravity fields
			draw(phyzFields[i],x,y,body:getAngle(),1,1,phyzFields[i]:getWidth()/2,phyzFields[i]:getHeight()/2)
		end
		--Draw pre-rendered image
		draw(phyzImages[i],x,y,body:getAngle(),1,1,phyzImages[i]:getWidth()/2,phyzImages[i]:getHeight()/2)
	end
	local body = map.player.body
	local x,y = body:getWorldCenter()
	draw(map.player.texture,x,y,body:getAngle(),50/map.player.texture:getWidth(),50/map.player.texture:getHeight(),map.player.texture:getWidth()/2,map.player.texture:getHeight()/2)
	graph.pop()
	local compX = 30 + (love.graphics.getWidth()-60)*(gameSettings["Compass X"]/100)
	local compY = 30 + (love.graphics.getHeight()-60)*(gameSettings["Compass Y"]/100)
	draw(compass,compX,compY,-body:getAngle(),1,1,compass:getWidth()/2,compass:getHeight()/2)
end
--===========================================================================================================Reset textures
function resetTextures()
	textures = {}
	phyzImages = {}
	phyzFields = {}
end